+++
title = "On-screen keyboard"
weight = 100
toc = true
outputs = [ "html", "pdf-in",]
date = "2021-08-23"
+++

# Introduction

Apertis can be used with a touchscreen only, in this case the user will need an On-screen Keyboard to be able to enter information like password, URLs, messages, …

# Terminology and concepts

In Wayland, multiple protocols are involved to allow user to enter a text.

## Text input

**text-input** protocol takes text from the compositor, and gives it to applications.
It lets applications tell the compositor when and what kind of text they need.
The protocol doesn't worry about the user, instead leaving that to the compositor.

![text-input](/images/onscreen_kbd_text_input.svg)

## Input method

**input-method** protocol allows the compositor to delegate work to let user input
the text to some other program.

![input-method](/images/onscreen_kbd_input_method.svg)

This protocol is very similar to *text-input*, because it lets *a program* send text
*to the compositor*, and allows *the compositor* to tell *the program* what kind of
text is needed. Notice the inversion! This time, the program is special. It communicates
with the user, and then gives the text to the compositor. The compositor is then
typically going to send the text onward to the currently focused application using
*text-input*, creating a chain: special program → compositor → focused application.

![input-method to focused application](/images/onscreen_kbd_focused.svg)

This protocol has place for some additional responsibilities, too. Because there is
typically only one application using this protocol, it can do things which would not
work with multiple applications. One of them is grabbing the keyboard, known to CJKV
language users. *Input-method* allows the special program to ask the compositor to send
it all keyboard presses ("exclusive grab"). Taking over the keyboard makes it possible
to send the text "你好" when "nihao" is typed. The other extension is meant for creating
a special pop up window, which the compositor places next to the text field, and which
can be used to show typing completions.

## Virtual keyboard

**virtual-keyboard** protocol is designed for programs which want to tell the compositor
to issue "fake" keyboard button press events.

Input methods support would have been complete here, if all we cared about was text.
However, the world is not so simple, and we have to deal with additional categories of
input before being useful:
- text in legacy applications which don't support text-input, and
- triggering actions which would normally need a keyboard.

This issue can be addressed by emulating button presses.

# Use cases

- A user wants to enter a text using the touchscreen.

# Non-use cases

# Requirements

The chosen approach must:
- allow to configure the keyboard layout
- be automatically enabled when user selects a text input zone
- not need any change to the applications

# Depending components

# Approach

A fully-fledged input method program will be a Wayland client using the *input-method*
protocol for submitting text, but also supporting *virtual-keyboard* for submitting
actions, and as a fallback for legacy applications.

![input-method + virtual-keyboard](/images/onscreen_kbd_virtual_keyboard.svg)

A compositor would ferry text around between the input method program and whichever
application is focused. It would also carry synthetic keyboard events from the input
method program to the focused application.

An application consuming text would support *text-input*, generally through a GUI
framework like GTK or Qt, and it would send enable and disable events whenever a text
input field comes into focus or becomes unfocused.
It would also accept keyboard events.

Legacy applications won't send enable and disable, even when a text field is focused,
and the user ready to type. When that happens, the compositor and the input method
won't realize that text should be submitted now. If the input method uses an on-screen
keyboard, it could remain hidden! Because of that, it's best to always make sure the
user can bring up the input method and input text, which would then be delivered as
keyboard events (which are always supported by applications).

Currently the majority of the on-screen keyboard applications was developped for `X11`
display server. Regarding `Wayland` display server only a few of them exist:
- weston-keyboard
- Maliit Keyboard 2
- Squeekboard

All of those on-screen keyboard implementation are GPLv3 or LGPLv3.

# Evaluation Report

## Gnome on-screen keyboard

On Debian, the onscreen keyboard is provided by [Caribou](https://wiki.gnome.org/Projects/Caribou), but is limited to `X11` or `XWayland` display servers.

|             | text-input | input-method | virtual-keyboard |
| ----------- | :--------: | :----------: | :--------------: |
| required    | -          | -            | -                |
| recommended | -          | -            | -                |

- License: LGPLv2.1
- Languages: Vala, Python

## Qt Virtual keyboard

Qt Virtual keyboard seems to have been deprecated in favor of application supporting
the Wayland input method protocols, see [this blogpost](https://blog.martin-graesslin.com/blog/2021/03/using-maliit-keyboard-in-a-plasma-wayland-session/).

## weston-keyboard

Simple implementation of an on-screen keyboard.
The application only support roman, numeric and arabic keyboards, which are hardcoded. As it graphical look.

|             | text-input | input-method | virtual-keyboard |
| ----------- | :--------: | :----------: | :--------------: |
| required    | v1         | v1           | -                |
| recommended | -          | -            | -                |

- License: X11, MIT and CC-BY-SA
- Languages: C

## Mallit Keyboard 2

Mallit Keyboard 2 is an evolution of the *Ubuntu Keyboard* plugin for Mallit, which can be run standalone using the Wayland input-method protocol.

|             | text-input | input-method | virtual-keyboard |
| ----------- | :--------: | :----------: | :--------------: |
| required    | -          | v1           | -                |
| recommended | -          | -            | -                |

- License: LGPLv3, BSD and CC-BY
- Languages: QML, C++

## Squeekboard

Squeekboard has been developped to be the on-screen keyboard of Librem 5 phone OS,
using [Phoc](https://gitlab.gnome.org/World/Phosh/phoc) compositor which is based
on [wlroots](https://github.com/swaywm/wlroots).

|             | text-input | input-method | virtual-keyboard |
| ----------- | :--------: | :----------: | :--------------: |
| required    | -          | -            | v1               |
| recommended | -          | v2           | -                |

- License: GPLv3
- Languages: Rust, C

## Weston

Weston is the Wayland compositor used in Apertis.

|             | text-input | input-method | virtual-keyboard |
| ----------- | :--------: | :----------: | :--------------: |
| current     | v1         | v1           | -                |

A [merge request](https://gitlab.freedesktop.org/wayland/weston/-/merge_requests/150)
exists to integrate *text-input* v3 and *input-method* v2 to Weston.

## GTK

|             | text-input | input-method | virtual-keyboard |
| ----------- | :--------: | :----------: | :--------------: |
| current     | v3 (since GTK3.24) | -    | -                |

## Qt

|             | text-input | input-method | virtual-keyboard |
| ----------- | :--------: | :----------: | :--------------: |
| current     | v2 (since Qt 5.9) | -     | -                |

# Recommendations

As the only GPLv3 free on-screen keyboard application is a simple/demo version, Apertis may have to improve it or use one of the existing one.

The former solution will need to develop, debug and support an application.

While the latter one will embedded a GPLv3 application which only interacts with the Weston compositor.

The approach could then be:

1. Patch Weston with latest protocol versions, i.e. *text-input* v3 and *input-method* v2.
1. Create patch for Weston to support *virtual-keyboard* protocol.
1. Add *Squeekboard* as a GPLv3 execption in Apertis.

# Risks

The different Wayland protocols involved in an on-screen keybopard are currently under
development, see [Stalled Upstream Protocol Work](https://gitlab.freedesktop.org/wayland/wayland-protocols/-/issues/39#stalled-upstream-protocol-work).

Those protocols needs to be integrated to Weston, see this Weston [MR](https://gitlab.freedesktop.org/wayland/weston/-/merge_requests/150).

# References

This documentation and some of the illustrations are based on or come from:
- [Wayland and input methods](https://dcz_self.gitlab.io/posts/input_method/) blog post
- [It's not about keyboards](https://dcz_self.gitlab.io/posts/not_keyboard/) blog post
- [Input Method Hub](https://gitlab.freedesktop.org/wayland/wayland-protocols/-/issues/39) Wayland issue
